/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1. 0  */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <process.h>
#include <fcntl.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/wait.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

extern char drive[20];
extern char form[20];
//extern char **aenv;
extern int NUM_TRACK;
extern char **environ;

int do_format(void);

PtWorkProcId_t	*bkgd_proc;
char msg[80];
PtArg_t arg[3];
int pd_in[2];
int pd_out[2];
pid_t child_pid;
char buf_in[100];
int len_in=100;
int pos=0;
int rc,flags,i,ti;
char tmp[80];
char iov[10];
char *fargs[3];
char pctge[20];
PhPoint_t pcbox;


int
format_drive( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	char *env_list[]={NULL};

	////////////////////////////////////////////////////////////////////
	//Malloc space for IO vectors

	for(rc=0;rc<3;rc++)
		fargs[rc]=(char *)malloc(20);

	/////////////////////////////////////////////////////////////////////
	//setup IO vectors

	strcpy(fargs[0],form);
	strcpy(fargs[1],drive);
	fargs[2]=NULL;

	//////////////////////////////////////////////////////////////////////
	//setup PCbox measurements

	pcbox.y=19;
	pcbox.x=0;


	start:

	///////////////////////////////////////////////////////////////////////
	//Set up the pipes that we'll need when we spawn fdformat

	if(pipe(pd_in)==-1)
		{
		sprintf(tmp,"Pipe in error: %s",strerror(errno));
		PtAskQuestion(ABW_Format_floppy,"Error Formatting Floppy",tmp,"helv14",
			"&OK",NULL,NULL,1);
		goto format_exit;
		}
	if(pipe(pd_out)==-1)
		{
		sprintf(tmp,"Pipe out error: %s",strerror(errno));
		PtAskQuestion(ABW_Format_floppy,"Error Formatting Floppy",tmp,"helv14",
			"&OK",NULL,NULL,1);
		goto format_exit;
		}

	/////////////////////////////////////////////////////////////////////
	//Now we'll set up the IO Vectors that we actually need

	iov[0]=(char)pd_out[0];
	iov[1]=(char)pd_in[1];
	iov[2]=(char)pd_in[1];
	iov[3]=(char)-1;
	iov[4]=(char)-1;
	iov[5]=(char)-1;
	iov[6]=(char)-1;
	iov[7]=(char)-1;
	iov[8]=(char)-1;
	iov[9]=(char)-1;

	///////////////////////////////////////////////////////////////////////
	//Prompt the user to make sure the disk is in the drive

	sprintf(tmp,"Put a new disk in drive: %s",drive);
	switch(PtAskQuestion(ABW_Format_floppy,"Final Check",tmp,"helv14","&OK","&Cancel",
		NULL,1))
		{
		case 2:
			goto format_exit;
			break;
		case 1:
		default:
			break;
		}

	i=0;

	/////////////////////////////////////////////////////////////////////
	//Set the correct resources on the status box

	PtSetArg(&arg[0],Pt_ARG_FILL_COLOR,Pg_BLUE,0);
	PtSetArg(&arg[1],Pt_ARG_BORDER_WIDTH,2,0);
	PtSetResources(ABW_boxfill,2,arg);

	/////////////////////////////////////////////////////////////////
	//Spawn the fdformat system command giving it our pipes as stdout and
	//stderr. We'll also pass it an environment to work with - default really.

	child_pid=qnx_spawn(0,NULL,0,-1,-1,0,"/bin/fdformat",fargs,environ,iov,-1);
	if(child_pid==-1)
		{
		sprintf(tmp,"Error Spawning Format:\n%s",strerror(errno));
		PtAskQuestion(ABW_Format_floppy,"Format Floppy",tmp,"helv14","OK",NULL,NULL,
			1);
		goto format_exit;
		}

	///////////////////////////////////////////////////////////////////
	//Close the open FDs we no longer need - just to keep it tidy

	close(pd_in[1]);
	close(pd_out[0]);

	//////////////////////////////////////////////////////////////////
	//Set O_NON_BLOCK flag so we get a stream of data from child

	flags=fcntl(pd_in[0],F_GETFL);
	if(flags==-1)
		{
		sprintf(tmp,"Error Setting O_NONBLOCK:\n%s",strerror(errno));
		PtAskQuestion(ABW_Format_floppy,"Format Floppy",tmp,"helv14","&OK",NULL,NULL,
			1);
		goto format_exit;
		}
	flags|=O_NONBLOCK;
	flags=fcntl(pd_in[0],F_SETFL,flags);
	if(flags==-1)
		{
		sprintf(tmp,"Error Setting O_NONBLOCK:\n%s",strerror(errno));
		PtAskQuestion(ABW_Format_floppy,"Format Floppy",tmp,"helv14","&OK",NULL,NULL,
			1);
		goto format_exit;
		}

	//////////////////////////////////////////////////////////////////////
	//Read data in from the child's output and count the lines and use them
	//as the basis for determining the % formatted.

	for(;;)
		{
		while(rc=read(pd_in[0],&buf_in[pos],len_in-pos))
			{
			if(rc==-1)
				break;
			pos+=rc;
			}
		if(pos)
			{
			buf_in[pos]=0;
			i++;
			ti=((int)(i*100/NUM_TRACK));
			strcpy(tmp,itoa(ti,pctge,10));

	///////////////////////////////////////////////////////////////////////
	//Update numerical widget

			PtSetArg(&arg[0],Pt_ARG_TEXT_STRING,tmp,0);
			PtSetResources(ABW_pc,1,&arg[0]);
			PtFlush();

	////////////////////////////////////////////////////////////////////
	//Update sliding bar widget

			pcbox.x=(short)(ti*2.36);
			PtSetArg(&arg[0],Pt_ARG_DIM,&pcbox,0);
			PtSetResources(ABW_boxfill,1,&arg[0]);
			PtFlush();

			pos=0;
			}

	////////////////////////////////////////////////////////////////////
	//Just check to see if fdformat has finished

		if(waitpid(child_pid,NULL,WNOHANG)!=0)
			break;
		}

	//////////////////////////////////////////////////////////////////////
	//Close the rest of the FDs

	close(pd_out[1]);
	close(pd_in[0]);

	////////////////////////////////////////////////////////////////////
	//Now dinit the drive

	sprintf(tmp,"dinit %s",drive);
	system(tmp);

	//////////////////////////////////////////////////////////////////////
	//Clean up on screen

	if(PtAskQuestion(ABW_Format_floppy,"Format Floppy","Format Another?",
		"helv14","&Yes","&No",NULL,1)==1)
		goto start;
	else
		exit(1);
		
	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	format_exit:
	return( Pt_CONTINUE );
	

	}


